#include "src/libpriv/rpmostree-cxxrs-prelude.h"
#include "rpmostree-cxxrs.h"
#include <algorithm>
#include <array>
#include <cstddef>
#include <cstdint>
#include <exception>
#include <initializer_list>
#include <iterator>
#include <new>
#include <string>
#include <type_traits>
#include <utility>

namespace rust {
inline namespace cxxbridge1 {
// #include "rust/cxx.h"

#ifndef CXXBRIDGE1_PANIC
#define CXXBRIDGE1_PANIC
template <typename Exception>
void panic [[noreturn]] (const char *msg);
#endif // CXXBRIDGE1_PANIC

struct unsafe_bitcopy_t;

namespace {
template <typename T>
class impl;
} // namespace

#ifndef CXXBRIDGE1_RUST_STRING
#define CXXBRIDGE1_RUST_STRING
class String final {
public:
  String() noexcept;
  String(const String &) noexcept;
  String(String &&) noexcept;
  ~String() noexcept;

  String(const std::string &);
  String(const char *);
  String(const char *, std::size_t);

  String &operator=(const String &) noexcept;
  String &operator=(String &&) noexcept;

  explicit operator std::string() const;

  const char *data() const noexcept;
  std::size_t size() const noexcept;
  std::size_t length() const noexcept;

  const char *c_str() noexcept;

  using iterator = char *;
  iterator begin() noexcept;
  iterator end() noexcept;

  using const_iterator = const char *;
  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;
  const_iterator cbegin() const noexcept;
  const_iterator cend() const noexcept;

  bool operator==(const String &) const noexcept;
  bool operator!=(const String &) const noexcept;
  bool operator<(const String &) const noexcept;
  bool operator<=(const String &) const noexcept;
  bool operator>(const String &) const noexcept;
  bool operator>=(const String &) const noexcept;

  String(unsafe_bitcopy_t, const String &) noexcept;

private:
  std::array<std::uintptr_t, 3> repr;
};
#endif // CXXBRIDGE1_RUST_STRING

#ifndef CXXBRIDGE1_RUST_STR
#define CXXBRIDGE1_RUST_STR
class Str final {
public:
  Str() noexcept;
  Str(const String &) noexcept;
  Str(const std::string &);
  Str(const char *);
  Str(const char *, std::size_t);

  Str &operator=(const Str &) noexcept = default;

  explicit operator std::string() const;

  const char *data() const noexcept;
  std::size_t size() const noexcept;
  std::size_t length() const noexcept;

  Str(const Str &) noexcept = default;
  ~Str() noexcept = default;

  using iterator = const char *;
  using const_iterator = const char *;
  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;
  const_iterator cbegin() const noexcept;
  const_iterator cend() const noexcept;

  bool operator==(const Str &) const noexcept;
  bool operator!=(const Str &) const noexcept;
  bool operator<(const Str &) const noexcept;
  bool operator<=(const Str &) const noexcept;
  bool operator>(const Str &) const noexcept;
  bool operator>=(const Str &) const noexcept;

private:
  friend impl<Str>;
  const char *ptr;
  std::size_t len;
};

inline const char *Str::data() const noexcept { return this->ptr; }

inline std::size_t Str::size() const noexcept { return this->len; }

inline std::size_t Str::length() const noexcept { return this->len; }
#endif // CXXBRIDGE1_RUST_STR

#ifndef CXXBRIDGE1_RUST_SLICE
#define CXXBRIDGE1_RUST_SLICE
namespace detail {
template <bool>
struct copy_assignable_if {};

template <>
struct copy_assignable_if<false> {
  copy_assignable_if() noexcept = default;
  copy_assignable_if(const copy_assignable_if &) noexcept = default;
  copy_assignable_if &operator=(const copy_assignable_if &) noexcept = delete;
  copy_assignable_if &operator=(copy_assignable_if &&) noexcept = default;
};
} // namespace detail

template <typename T>
class Slice final
    : private detail::copy_assignable_if<std::is_const<T>::value> {
public:
  Slice() noexcept;
  Slice(T *, std::size_t count) noexcept;

  Slice &operator=(const Slice<T> &) noexcept = default;
  Slice &operator=(Slice<T> &&) noexcept = default;

  T *data() const noexcept;
  std::size_t size() const noexcept;
  std::size_t length() const noexcept;

  Slice(const Slice<T> &) noexcept = default;
  ~Slice() noexcept = default;

  class iterator;
  iterator begin() const noexcept;
  iterator end() const noexcept;

private:
  friend impl<Slice>;
  T *ptr;
  std::size_t len;
};

template <typename T>
class Slice<T>::iterator final {
public:
  using difference_type = std::ptrdiff_t;
  using value_type = T;
  using pointer = typename std::add_pointer<T>::type;
  using reference = typename std::add_lvalue_reference<T>::type;
  using iterator_category = std::forward_iterator_tag;

  T &operator*() const noexcept;
  T *operator->() const noexcept;
  iterator &operator++() noexcept;
  iterator operator++(int) noexcept;
  bool operator==(const iterator &) const noexcept;
  bool operator!=(const iterator &) const noexcept;

private:
  friend class Slice;
  T *pos;
};

template <typename T>
Slice<T>::Slice() noexcept : ptr(reinterpret_cast<T *>(alignof(T))), len(0) {}

template <typename T>
Slice<T>::Slice(T *s, std::size_t count) noexcept : ptr(s), len(count) {}

template <typename T>
T *Slice<T>::data() const noexcept {
  return this->ptr;
}

template <typename T>
std::size_t Slice<T>::size() const noexcept {
  return this->len;
}

template <typename T>
std::size_t Slice<T>::length() const noexcept {
  return this->len;
}

template <typename T>
T &Slice<T>::iterator::operator*() const noexcept {
  return *this->pos;
}

template <typename T>
T *Slice<T>::iterator::operator->() const noexcept {
  return this->pos;
}

template <typename T>
typename Slice<T>::iterator &Slice<T>::iterator::operator++() noexcept {
  ++this->pos;
  return *this;
}

template <typename T>
typename Slice<T>::iterator Slice<T>::iterator::operator++(int) noexcept {
  auto ret = iterator(*this);
  ++this->pos;
  return ret;
}

template <typename T>
bool Slice<T>::iterator::operator==(const iterator &other) const noexcept {
  return this->pos == other.pos;
}

template <typename T>
bool Slice<T>::iterator::operator!=(const iterator &other) const noexcept {
  return this->pos != other.pos;
}

template <typename T>
typename Slice<T>::iterator Slice<T>::begin() const noexcept {
  iterator it;
  it.pos = this->ptr;
  return it;
}

template <typename T>
typename Slice<T>::iterator Slice<T>::end() const noexcept {
  iterator it = this->begin();
  it.pos += this->len;
  return it;
}
#endif // CXXBRIDGE1_RUST_SLICE

#ifndef CXXBRIDGE1_RUST_BOX
#define CXXBRIDGE1_RUST_BOX
template <typename T>
class Box final {
public:
  using value_type = T;
  using const_pointer =
      typename std::add_pointer<typename std::add_const<T>::type>::type;
  using pointer = typename std::add_pointer<T>::type;

  Box() = delete;
  Box(const Box &);
  Box(Box &&) noexcept;
  ~Box() noexcept;

  explicit Box(const T &);
  explicit Box(T &&);

  Box &operator=(const Box &);
  Box &operator=(Box &&) noexcept;

  const T *operator->() const noexcept;
  const T &operator*() const noexcept;
  T *operator->() noexcept;
  T &operator*() noexcept;

  template <typename... Fields>
  static Box in_place(Fields &&...);

  static Box from_raw(T *) noexcept;

  T *into_raw() noexcept;

private:
  class uninit;
  class allocation;
  Box(uninit) noexcept;
  void drop() noexcept;
  T *ptr;
};

template <typename T>
class Box<T>::uninit {};

template <typename T>
class Box<T>::allocation {
  static T *alloc() noexcept;
  static void dealloc(T *) noexcept;

public:
  allocation() noexcept : ptr(alloc()) {}
  ~allocation() noexcept {
    if (this->ptr) {
      dealloc(this->ptr);
    }
  }
  T *ptr;
};

template <typename T>
Box<T>::Box(const Box &other) : Box(*other) {}

template <typename T>
Box<T>::Box(Box &&other) noexcept : ptr(other.ptr) {
  other.ptr = nullptr;
}

template <typename T>
Box<T>::Box(const T &val) {
  allocation alloc;
  ::new (alloc.ptr) T(val);
  this->ptr = alloc.ptr;
  alloc.ptr = nullptr;
}

template <typename T>
Box<T>::Box(T &&val) {
  allocation alloc;
  ::new (alloc.ptr) T(std::move(val));
  this->ptr = alloc.ptr;
  alloc.ptr = nullptr;
}

template <typename T>
Box<T>::~Box() noexcept {
  if (this->ptr) {
    this->drop();
  }
}

template <typename T>
Box<T> &Box<T>::operator=(const Box &other) {
  if (this->ptr) {
    **this = *other;
  } else {
    allocation alloc;
    ::new (alloc.ptr) T(*other);
    this->ptr = alloc.ptr;
    alloc.ptr = nullptr;
  }
  return *this;
}

template <typename T>
Box<T> &Box<T>::operator=(Box &&other) noexcept {
  if (this->ptr) {
    this->drop();
  }
  this->ptr = other.ptr;
  other.ptr = nullptr;
  return *this;
}

template <typename T>
const T *Box<T>::operator->() const noexcept {
  return this->ptr;
}

template <typename T>
const T &Box<T>::operator*() const noexcept {
  return *this->ptr;
}

template <typename T>
T *Box<T>::operator->() noexcept {
  return this->ptr;
}

template <typename T>
T &Box<T>::operator*() noexcept {
  return *this->ptr;
}

template <typename T>
template <typename... Fields>
Box<T> Box<T>::in_place(Fields &&... fields) {
  allocation alloc;
  auto ptr = alloc.ptr;
  ::new (ptr) T{std::forward<Fields>(fields)...};
  alloc.ptr = nullptr;
  return from_raw(ptr);
}

template <typename T>
Box<T> Box<T>::from_raw(T *raw) noexcept {
  Box box = uninit{};
  box.ptr = raw;
  return box;
}

template <typename T>
T *Box<T>::into_raw() noexcept {
  T *raw = this->ptr;
  this->ptr = nullptr;
  return raw;
}

template <typename T>
Box<T>::Box(uninit) noexcept {}
#endif // CXXBRIDGE1_RUST_BOX

#ifndef CXXBRIDGE1_RUST_BITCOPY
#define CXXBRIDGE1_RUST_BITCOPY
struct unsafe_bitcopy_t final {
  explicit unsafe_bitcopy_t() = default;
};

constexpr unsafe_bitcopy_t unsafe_bitcopy{};
#endif // CXXBRIDGE1_RUST_BITCOPY

#ifndef CXXBRIDGE1_RUST_VEC
#define CXXBRIDGE1_RUST_VEC
template <typename T>
class Vec final {
public:
  using value_type = T;

  Vec() noexcept;
  Vec(std::initializer_list<T>);
  Vec(const Vec &);
  Vec(Vec &&) noexcept;
  ~Vec() noexcept;

  Vec &operator=(Vec &&) noexcept;
  Vec &operator=(const Vec &);

  std::size_t size() const noexcept;
  bool empty() const noexcept;
  const T *data() const noexcept;
  T *data() noexcept;
  std::size_t capacity() const noexcept;

  const T &operator[](std::size_t n) const noexcept;
  const T &at(std::size_t n) const;
  const T &front() const;
  const T &back() const;

  T &operator[](std::size_t n) noexcept;
  T &at(std::size_t n);
  T &front();
  T &back();

  void reserve(std::size_t new_cap);
  void push_back(const T &value);
  void push_back(T &&value);
  template <typename... Args>
  void emplace_back(Args &&... args);

  class iterator;
  iterator begin() noexcept;
  iterator end() noexcept;

  using const_iterator = typename Vec<const T>::iterator;
  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;
  const_iterator cbegin() const noexcept;
  const_iterator cend() const noexcept;

  Vec(unsafe_bitcopy_t, const Vec &) noexcept;

private:
  static std::size_t stride() noexcept;
  void reserve_total(std::size_t cap) noexcept;
  void set_len(std::size_t len) noexcept;
  void drop() noexcept;

  std::array<std::uintptr_t, 3> repr;
};

template <typename T>
class Vec<T>::iterator final {
public:
  using iterator_category = std::random_access_iterator_tag;
  using value_type = T;
  using difference_type = std::ptrdiff_t;
  using pointer = typename std::add_pointer<T>::type;
  using reference = typename std::add_lvalue_reference<T>::type;

  reference operator*() const noexcept;
  pointer operator->() const noexcept;
  reference operator[](difference_type) const noexcept;

  iterator &operator++() noexcept;
  iterator operator++(int) noexcept;
  iterator &operator--() noexcept;
  iterator operator--(int) noexcept;

  iterator &operator+=(difference_type) noexcept;
  iterator &operator-=(difference_type) noexcept;
  iterator operator+(difference_type) const noexcept;
  iterator operator-(difference_type) const noexcept;
  difference_type operator-(const iterator &) const noexcept;

  bool operator==(const iterator &) const noexcept;
  bool operator!=(const iterator &) const noexcept;
  bool operator<(const iterator &) const noexcept;
  bool operator>(const iterator &) const noexcept;
  bool operator<=(const iterator &) const noexcept;
  bool operator>=(const iterator &) const noexcept;

private:
  friend class Vec;
  friend class Vec<typename std::remove_const<T>::type>;
  void *pos;
  std::size_t stride;
};

template <typename T>
Vec<T>::Vec(std::initializer_list<T> init) : Vec{} {
  this->reserve_total(init.size());
  std::move(init.begin(), init.end(), std::back_inserter(*this));
}

template <typename T>
Vec<T>::Vec(const Vec &other) : Vec() {
  this->reserve_total(other.size());
  std::copy(other.begin(), other.end(), std::back_inserter(*this));
}

template <typename T>
Vec<T>::Vec(Vec &&other) noexcept : repr(other.repr) {
  new (&other) Vec();
}

template <typename T>
Vec<T>::~Vec() noexcept {
  this->drop();
}

template <typename T>
Vec<T> &Vec<T>::operator=(Vec &&other) noexcept {
  if (this != &other) {
    this->drop();
    this->repr = other.repr;
    new (&other) Vec();
  }
  return *this;
}

template <typename T>
Vec<T> &Vec<T>::operator=(const Vec &other) {
  if (this != &other) {
    this->drop();
    new (this) Vec(other);
  }
  return *this;
}

template <typename T>
bool Vec<T>::empty() const noexcept {
  return size() == 0;
}

template <typename T>
T *Vec<T>::data() noexcept {
  return const_cast<T *>(const_cast<const Vec<T> *>(this)->data());
}

template <typename T>
const T &Vec<T>::operator[](std::size_t n) const noexcept {
  auto data = reinterpret_cast<const char *>(this->data());
  return *reinterpret_cast<const T *>(data + n * this->stride());
}

template <typename T>
const T &Vec<T>::at(std::size_t n) const {
  if (n >= this->size()) {
    panic<std::out_of_range>("rust::Vec index out of range");
  }
  return (*this)[n];
}

template <typename T>
const T &Vec<T>::front() const {
  return (*this)[0];
}

template <typename T>
const T &Vec<T>::back() const {
  return (*this)[this->size() - 1];
}

template <typename T>
T &Vec<T>::operator[](std::size_t n) noexcept {
  auto data = reinterpret_cast<char *>(this->data());
  return *reinterpret_cast<T *>(data + n * this->stride());
}

template <typename T>
T &Vec<T>::at(std::size_t n) {
  if (n >= this->size()) {
    panic<std::out_of_range>("rust::Vec index out of range");
  }
  return (*this)[n];
}

template <typename T>
T &Vec<T>::front() {
  return (*this)[0];
}

template <typename T>
T &Vec<T>::back() {
  return (*this)[this->size() - 1];
}

template <typename T>
void Vec<T>::reserve(std::size_t new_cap) {
  this->reserve_total(new_cap);
}

template <typename T>
void Vec<T>::push_back(const T &value) {
  this->emplace_back(value);
}

template <typename T>
void Vec<T>::push_back(T &&value) {
  this->emplace_back(std::move(value));
}

template <typename T>
template <typename... Args>
void Vec<T>::emplace_back(Args &&... args) {
  auto size = this->size();
  this->reserve_total(size + 1);
  ::new (reinterpret_cast<T *>(reinterpret_cast<char *>(this->data()) +
                               size * this->stride()))
      T(std::forward<Args>(args)...);
  this->set_len(size + 1);
}

template <typename T>
typename Vec<T>::iterator::reference
Vec<T>::iterator::operator*() const noexcept {
  return *static_cast<T *>(this->pos);
}

template <typename T>
typename Vec<T>::iterator::pointer
Vec<T>::iterator::operator->() const noexcept {
  return static_cast<T *>(this->pos);
}

template <typename T>
typename Vec<T>::iterator::reference Vec<T>::iterator::operator[](
    typename Vec<T>::iterator::difference_type n) const noexcept {
  auto pos = static_cast<char *>(this->pos) + this->stride * n;
  return *static_cast<T *>(pos);
}

template <typename T>
typename Vec<T>::iterator &Vec<T>::iterator::operator++() noexcept {
  this->pos = static_cast<char *>(this->pos) + this->stride;
  return *this;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::iterator::operator++(int) noexcept {
  auto ret = iterator(*this);
  this->pos = static_cast<char *>(this->pos) + this->stride;
  return ret;
}

template <typename T>
typename Vec<T>::iterator &Vec<T>::iterator::operator--() noexcept {
  this->pos = static_cast<char *>(this->pos) - this->stride;
  return *this;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::iterator::operator--(int) noexcept {
  auto ret = iterator(*this);
  this->pos = static_cast<char *>(this->pos) - this->stride;
  return ret;
}

template <typename T>
typename Vec<T>::iterator &Vec<T>::iterator::operator+=(
    typename Vec<T>::iterator::difference_type n) noexcept {
  this->pos = static_cast<char *>(this->pos) + this->stride * n;
  return *this;
}

template <typename T>
typename Vec<T>::iterator &Vec<T>::iterator::operator-=(
    typename Vec<T>::iterator::difference_type n) noexcept {
  this->pos = static_cast<char *>(this->pos) - this->stride * n;
  return *this;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::iterator::operator+(
    typename Vec<T>::iterator::difference_type n) const noexcept {
  auto ret = iterator(*this);
  ret.pos = static_cast<char *>(this->pos) + this->stride * n;
  return ret;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::iterator::operator-(
    typename Vec<T>::iterator::difference_type n) const noexcept {
  auto ret = iterator(*this);
  ret.pos = static_cast<char *>(this->pos) - this->stride * n;
  return ret;
}

template <typename T>
typename Vec<T>::iterator::difference_type
Vec<T>::iterator::operator-(const iterator &other) const noexcept {
  auto diff = std::distance(static_cast<char *>(other.pos),
                            static_cast<char *>(this->pos));
  return diff / this->stride;
}

template <typename T>
bool Vec<T>::iterator::operator==(const iterator &other) const noexcept {
  return this->pos == other.pos;
}

template <typename T>
bool Vec<T>::iterator::operator!=(const iterator &other) const noexcept {
  return this->pos != other.pos;
}

template <typename T>
bool Vec<T>::iterator::operator>(const iterator &other) const noexcept {
  return this->pos > other.pos;
}

template <typename T>
bool Vec<T>::iterator::operator<(const iterator &other) const noexcept {
  return this->pos < other.pos;
}

template <typename T>
bool Vec<T>::iterator::operator>=(const iterator &other) const noexcept {
  return this->pos >= other.pos;
}

template <typename T>
bool Vec<T>::iterator::operator<=(const iterator &other) const noexcept {
  return this->pos <= other.pos;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::begin() noexcept {
  iterator it;
  it.pos = const_cast<typename std::remove_const<T>::type *>(this->data());
  it.stride = this->stride();
  return it;
}

template <typename T>
typename Vec<T>::iterator Vec<T>::end() noexcept {
  iterator it = this->begin();
  it.pos = static_cast<char *>(it.pos) + it.stride * this->size();
  return it;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::begin() const noexcept {
  return this->cbegin();
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::end() const noexcept {
  return this->cend();
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::cbegin() const noexcept {
  const_iterator it;
  it.pos = const_cast<typename std::remove_const<T>::type *>(this->data());
  it.stride = this->stride();
  return it;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::cend() const noexcept {
  const_iterator it = this->cbegin();
  it.pos = static_cast<char *>(it.pos) + it.stride * this->size();
  return it;
}

template <typename T>
Vec<T>::Vec(unsafe_bitcopy_t, const Vec &bits) noexcept : repr(bits.repr) {}
#endif // CXXBRIDGE1_RUST_VEC

#ifndef CXXBRIDGE1_RUST_ERROR
#define CXXBRIDGE1_RUST_ERROR
class Error final : public std::exception {
public:
  Error(const Error &);
  Error(Error &&) noexcept;
  ~Error() noexcept override;

  Error &operator=(const Error &);
  Error &operator=(Error &&) noexcept;

  const char *what() const noexcept override;

private:
  Error() noexcept = default;
  friend impl<Error>;
  const char *msg;
  std::size_t len;
};
#endif // CXXBRIDGE1_RUST_ERROR

#ifndef CXXBRIDGE1_RUST_OPAQUE
#define CXXBRIDGE1_RUST_OPAQUE
class Opaque {
public:
  Opaque() = delete;
  Opaque(const Opaque &) = delete;
  ~Opaque() = delete;
};
#endif // CXXBRIDGE1_RUST_OPAQUE

namespace detail {
template <typename T, typename = void *>
struct operator_new {
  void *operator()(::std::size_t sz) { return ::operator new(sz); }
};

template <typename T>
struct operator_new<T, decltype(T::operator new(sizeof(T)))> {
  void *operator()(::std::size_t sz) { return T::operator new(sz); }
};
} // namespace detail

template <typename T>
union ManuallyDrop {
  T value;
  ManuallyDrop(T &&value) : value(::std::move(value)) {}
  ~ManuallyDrop() {}
};

template <typename T>
union MaybeUninit {
  T value;
  void *operator new(::std::size_t sz) { return detail::operator_new<T>{}(sz); }
  MaybeUninit() {}
  ~MaybeUninit() {}
};

namespace {
namespace repr {
struct PtrLen final {
  void *ptr;
  ::std::size_t len;
};
} // namespace repr

template <>
class impl<Str> final {
public:
  static repr::PtrLen repr(Str str) noexcept {
    return repr::PtrLen{const_cast<char *>(str.ptr), str.len};
  }
};

template <typename T>
class impl<Slice<T>> final {
public:
  static Slice<T> slice(repr::PtrLen repr) noexcept {
    return {static_cast<T *>(repr.ptr), repr.len};
  }
  static repr::PtrLen repr(Slice<T> slice) noexcept {
    return repr::PtrLen{
        const_cast<typename ::std::remove_const<T>::type *>(slice.ptr),
        slice.len};
  }
};

template <>
class impl<Error> final {
public:
  static Error error(repr::PtrLen repr) noexcept {
    Error error;
    error.msg = static_cast<const char *>(repr.ptr);
    error.len = repr.len;
    return error;
  }
};
} // namespace
} // namespace cxxbridge1
} // namespace rust

namespace rpmostreecxx {
  struct TempEtcGuard;
  struct StringMapping;
  struct LiveApplyState;
  struct PasswdDB;
}

namespace rpmostreecxx {
#ifndef CXXBRIDGE1_STRUCT_rpmostreecxx$TempEtcGuard
#define CXXBRIDGE1_STRUCT_rpmostreecxx$TempEtcGuard
struct TempEtcGuard final : public ::rust::Opaque {
  void undo() const;
};
#endif // CXXBRIDGE1_STRUCT_rpmostreecxx$TempEtcGuard

#ifndef CXXBRIDGE1_STRUCT_rpmostreecxx$StringMapping
#define CXXBRIDGE1_STRUCT_rpmostreecxx$StringMapping
// Currently cxx-rs doesn't support mappings; like probably most projects,
// by far our most common case is a mapping from string -> string and since
// our data sizes aren't large, we serialize this as a vector of strings pairs.
struct StringMapping final {
  ::rust::String k;
  ::rust::String v;

  using IsRelocatable = ::std::true_type;
};
#endif // CXXBRIDGE1_STRUCT_rpmostreecxx$StringMapping

#ifndef CXXBRIDGE1_STRUCT_rpmostreecxx$LiveApplyState
#define CXXBRIDGE1_STRUCT_rpmostreecxx$LiveApplyState
// A copy of LiveFsState that is bridged to C++; the main
// change here is we can't use Option<> yet, so empty values
// are represented by the empty string.
struct LiveApplyState final {
  ::rust::String inprogress;
  ::rust::String commit;

  using IsRelocatable = ::std::true_type;
};
#endif // CXXBRIDGE1_STRUCT_rpmostreecxx$LiveApplyState

#ifndef CXXBRIDGE1_STRUCT_rpmostreecxx$PasswdDB
#define CXXBRIDGE1_STRUCT_rpmostreecxx$PasswdDB
struct PasswdDB final : public ::rust::Opaque {
  void add_user(::std::uint32_t uid, ::rust::Str username) noexcept;
  ::rust::String lookup_user(::std::uint32_t uid) const;
  void add_group(::std::uint32_t gid, ::rust::Str groupname) noexcept;
  ::rust::String lookup_group(::std::uint32_t gid) const;
  void add_group_content(::std::int32_t rootfs, ::rust::Str group_path);
  void add_passwd_content(::std::int32_t rootfs, ::rust::Str passwd_path);
};
#endif // CXXBRIDGE1_STRUCT_rpmostreecxx$PasswdDB

extern "C" {
::rust::repr::PtrLen rpmostreecxx$cxxbridge1$client_handle_fd_argument(::rust::repr::PtrLen arg, ::rust::repr::PtrLen arch, ::rust::Vec<::std::int32_t> *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$cliwrap_write_wrappers(::std::int32_t rootfs) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$cliwrap_entrypoint(::rust::Vec<::rust::String> *argv) noexcept;

void rpmostreecxx$cxxbridge1$cliwrap_destdir(::rust::String *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$prepare_tempetc_guard(::std::int32_t rootfs, ::rust::Box<::rpmostreecxx::TempEtcGuard> *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$TempEtcGuard$undo(const ::rpmostreecxx::TempEtcGuard &self) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$get_systemctl_wrapper() noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$compose_postprocess_final(::std::int32_t rootfs_dfd) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$get_dracut_random_cpio() noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$initramfs_overlay_generate(const ::rust::Vec<::rust::String> &files, ::rpmostreecxx::GCancellable &cancellable, ::std::int32_t *return$) noexcept;

void rpmostreecxx$cxxbridge1$journal_print_staging_failure() noexcept;

void rpmostreecxx$cxxbridge1$progress_begin_task(::rust::repr::PtrLen msg) noexcept;

void rpmostreecxx$cxxbridge1$progress_begin_n_items(::rust::repr::PtrLen msg, ::std::uint64_t n) noexcept;

void rpmostreecxx$cxxbridge1$progress_begin_percent(::rust::repr::PtrLen msg) noexcept;

void rpmostreecxx$cxxbridge1$progress_set_message(::rust::repr::PtrLen msg) noexcept;

void rpmostreecxx$cxxbridge1$progress_set_sub_message(::rust::repr::PtrLen msg) noexcept;

void rpmostreecxx$cxxbridge1$progress_update(::std::uint64_t n) noexcept;

void rpmostreecxx$cxxbridge1$progress_end(::rust::repr::PtrLen suffix) noexcept;

bool rpmostreecxx$cxxbridge1$script_is_ignored(::rust::repr::PtrLen pkg, ::rust::repr::PtrLen script) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$testutils_entrypoint(::rust::Vec<::rust::String> *argv) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$varsubstitute(::rust::repr::PtrLen s, const ::rust::Vec<::rpmostreecxx::StringMapping> &vars, ::rust::String *return$) noexcept;

void rpmostreecxx$cxxbridge1$get_features(::rust::Vec<::rust::String> *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$sealed_memfd(::rust::repr::PtrLen description, ::rust::repr::PtrLen content, ::std::int32_t *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$get_live_apply_state(::rpmostreecxx::OstreeSysroot &sysroot, ::rpmostreecxx::OstreeDeployment &deployment, ::rpmostreecxx::LiveApplyState *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$has_live_apply_state(::rpmostreecxx::OstreeSysroot &sysroot, ::rpmostreecxx::OstreeDeployment &deployment, bool *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$transaction_apply_live(::rpmostreecxx::OstreeSysroot &sysroot, ::rust::repr::PtrLen target) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$passwddb_open(::std::int32_t rootfs, ::rust::Box<::rpmostreecxx::PasswdDB> *return$) noexcept;

void rpmostreecxx$cxxbridge1$PasswdDB$add_user(::rpmostreecxx::PasswdDB &self, ::std::uint32_t uid, ::rust::repr::PtrLen username) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$PasswdDB$lookup_user(const ::rpmostreecxx::PasswdDB &self, ::std::uint32_t uid, ::rust::String *return$) noexcept;

void rpmostreecxx$cxxbridge1$PasswdDB$add_group(::rpmostreecxx::PasswdDB &self, ::std::uint32_t gid, ::rust::repr::PtrLen groupname) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$PasswdDB$lookup_group(const ::rpmostreecxx::PasswdDB &self, ::std::uint32_t gid, ::rust::String *return$) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$PasswdDB$add_group_content(::rpmostreecxx::PasswdDB &self, ::std::int32_t rootfs, ::rust::repr::PtrLen group_path) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$PasswdDB$add_passwd_content(::rpmostreecxx::PasswdDB &self, ::std::int32_t rootfs, ::rust::repr::PtrLen passwd_path) noexcept;

::rust::repr::PtrLen rpmostreecxx$cxxbridge1$countme_entrypoint(::rust::Vec<::rust::String> *argv) noexcept;
} // extern "C"

::rust::Vec<::std::int32_t> client_handle_fd_argument(::rust::Str arg, ::rust::Str arch) {
  ::rust::MaybeUninit<::rust::Vec<::std::int32_t>> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$client_handle_fd_argument(::rust::impl<::rust::Str>::repr(arg), ::rust::impl<::rust::Str>::repr(arch), &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void cliwrap_write_wrappers(::std::int32_t rootfs) {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$cliwrap_write_wrappers(rootfs);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

void cliwrap_entrypoint(::rust::Vec<::rust::String> argv) {
  ::rust::ManuallyDrop<::rust::Vec<::rust::String>> argv$(::std::move(argv));
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$cliwrap_entrypoint(&argv$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

::rust::String cliwrap_destdir() noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  rpmostreecxx$cxxbridge1$cliwrap_destdir(&return$.value);
  return ::std::move(return$.value);
}

::rust::Box<::rpmostreecxx::TempEtcGuard> prepare_tempetc_guard(::std::int32_t rootfs) {
  ::rust::MaybeUninit<::rust::Box<::rpmostreecxx::TempEtcGuard>> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$prepare_tempetc_guard(rootfs, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void TempEtcGuard::undo() const {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$TempEtcGuard$undo(*this);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

::rust::Slice<const ::std::uint8_t> get_systemctl_wrapper() noexcept {
  return ::rust::impl<::rust::Slice<const ::std::uint8_t>>::slice(rpmostreecxx$cxxbridge1$get_systemctl_wrapper());
}

void compose_postprocess_final(::std::int32_t rootfs_dfd) {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$compose_postprocess_final(rootfs_dfd);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

::rust::Slice<const ::std::uint8_t> get_dracut_random_cpio() noexcept {
  return ::rust::impl<::rust::Slice<const ::std::uint8_t>>::slice(rpmostreecxx$cxxbridge1$get_dracut_random_cpio());
}

::std::int32_t initramfs_overlay_generate(const ::rust::Vec<::rust::String> &files, ::rpmostreecxx::GCancellable &cancellable) {
  ::rust::MaybeUninit<::std::int32_t> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$initramfs_overlay_generate(files, cancellable, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void journal_print_staging_failure() noexcept {
  rpmostreecxx$cxxbridge1$journal_print_staging_failure();
}

void progress_begin_task(::rust::Str msg) noexcept {
  rpmostreecxx$cxxbridge1$progress_begin_task(::rust::impl<::rust::Str>::repr(msg));
}

void progress_begin_n_items(::rust::Str msg, ::std::uint64_t n) noexcept {
  rpmostreecxx$cxxbridge1$progress_begin_n_items(::rust::impl<::rust::Str>::repr(msg), n);
}

void progress_begin_percent(::rust::Str msg) noexcept {
  rpmostreecxx$cxxbridge1$progress_begin_percent(::rust::impl<::rust::Str>::repr(msg));
}

void progress_set_message(::rust::Str msg) noexcept {
  rpmostreecxx$cxxbridge1$progress_set_message(::rust::impl<::rust::Str>::repr(msg));
}

void progress_set_sub_message(::rust::Str msg) noexcept {
  rpmostreecxx$cxxbridge1$progress_set_sub_message(::rust::impl<::rust::Str>::repr(msg));
}

void progress_update(::std::uint64_t n) noexcept {
  rpmostreecxx$cxxbridge1$progress_update(n);
}

void progress_end(::rust::Str suffix) noexcept {
  rpmostreecxx$cxxbridge1$progress_end(::rust::impl<::rust::Str>::repr(suffix));
}

bool script_is_ignored(::rust::Str pkg, ::rust::Str script) noexcept {
  return rpmostreecxx$cxxbridge1$script_is_ignored(::rust::impl<::rust::Str>::repr(pkg), ::rust::impl<::rust::Str>::repr(script));
}

void testutils_entrypoint(::rust::Vec<::rust::String> argv) {
  ::rust::ManuallyDrop<::rust::Vec<::rust::String>> argv$(::std::move(argv));
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$testutils_entrypoint(&argv$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

::rust::String varsubstitute(::rust::Str s, const ::rust::Vec<::rpmostreecxx::StringMapping> &vars) {
  ::rust::MaybeUninit<::rust::String> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$varsubstitute(::rust::impl<::rust::Str>::repr(s), vars, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

::rust::Vec<::rust::String> get_features() noexcept {
  ::rust::MaybeUninit<::rust::Vec<::rust::String>> return$;
  rpmostreecxx$cxxbridge1$get_features(&return$.value);
  return ::std::move(return$.value);
}

::std::int32_t sealed_memfd(::rust::Str description, ::rust::Slice<const ::std::uint8_t> content) {
  ::rust::MaybeUninit<::std::int32_t> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$sealed_memfd(::rust::impl<::rust::Str>::repr(description), ::rust::impl<::rust::Slice<const ::std::uint8_t>>::repr(content), &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

::rpmostreecxx::LiveApplyState get_live_apply_state(::rpmostreecxx::OstreeSysroot &sysroot, ::rpmostreecxx::OstreeDeployment &deployment) {
  ::rust::MaybeUninit<::rpmostreecxx::LiveApplyState> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$get_live_apply_state(sysroot, deployment, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

bool has_live_apply_state(::rpmostreecxx::OstreeSysroot &sysroot, ::rpmostreecxx::OstreeDeployment &deployment) {
  ::rust::MaybeUninit<bool> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$has_live_apply_state(sysroot, deployment, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void transaction_apply_live(::rpmostreecxx::OstreeSysroot &sysroot, ::rust::Str target) {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$transaction_apply_live(sysroot, ::rust::impl<::rust::Str>::repr(target));
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

::rust::Box<::rpmostreecxx::PasswdDB> passwddb_open(::std::int32_t rootfs) {
  ::rust::MaybeUninit<::rust::Box<::rpmostreecxx::PasswdDB>> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$passwddb_open(rootfs, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void PasswdDB::add_user(::std::uint32_t uid, ::rust::Str username) noexcept {
  rpmostreecxx$cxxbridge1$PasswdDB$add_user(*this, uid, ::rust::impl<::rust::Str>::repr(username));
}

::rust::String PasswdDB::lookup_user(::std::uint32_t uid) const {
  ::rust::MaybeUninit<::rust::String> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$PasswdDB$lookup_user(*this, uid, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void PasswdDB::add_group(::std::uint32_t gid, ::rust::Str groupname) noexcept {
  rpmostreecxx$cxxbridge1$PasswdDB$add_group(*this, gid, ::rust::impl<::rust::Str>::repr(groupname));
}

::rust::String PasswdDB::lookup_group(::std::uint32_t gid) const {
  ::rust::MaybeUninit<::rust::String> return$;
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$PasswdDB$lookup_group(*this, gid, &return$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
  return ::std::move(return$.value);
}

void PasswdDB::add_group_content(::std::int32_t rootfs, ::rust::Str group_path) {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$PasswdDB$add_group_content(*this, rootfs, ::rust::impl<::rust::Str>::repr(group_path));
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

void PasswdDB::add_passwd_content(::std::int32_t rootfs, ::rust::Str passwd_path) {
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$PasswdDB$add_passwd_content(*this, rootfs, ::rust::impl<::rust::Str>::repr(passwd_path));
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}

void countme_entrypoint(::rust::Vec<::rust::String> argv) {
  ::rust::ManuallyDrop<::rust::Vec<::rust::String>> argv$(::std::move(argv));
  ::rust::repr::PtrLen error$ = rpmostreecxx$cxxbridge1$countme_entrypoint(&argv$.value);
  if (error$.ptr) {
    throw ::rust::impl<::rust::Error>::error(error$);
  }
}
} // namespace rpmostreecxx

extern "C" {
#ifndef CXXBRIDGE1_RUST_BOX_rpmostreecxx$TempEtcGuard
#define CXXBRIDGE1_RUST_BOX_rpmostreecxx$TempEtcGuard
::rpmostreecxx::TempEtcGuard *cxxbridge1$box$rpmostreecxx$TempEtcGuard$alloc() noexcept;
void cxxbridge1$box$rpmostreecxx$TempEtcGuard$dealloc(::rpmostreecxx::TempEtcGuard *) noexcept;
void cxxbridge1$box$rpmostreecxx$TempEtcGuard$drop(::rust::Box<::rpmostreecxx::TempEtcGuard> *ptr) noexcept;
#endif // CXXBRIDGE1_RUST_BOX_rpmostreecxx$TempEtcGuard

#ifndef CXXBRIDGE1_RUST_VEC_rpmostreecxx$StringMapping
#define CXXBRIDGE1_RUST_VEC_rpmostreecxx$StringMapping
void cxxbridge1$rust_vec$rpmostreecxx$StringMapping$new(const ::rust::Vec<::rpmostreecxx::StringMapping> *ptr) noexcept;
void cxxbridge1$rust_vec$rpmostreecxx$StringMapping$drop(::rust::Vec<::rpmostreecxx::StringMapping> *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$rpmostreecxx$StringMapping$len(const ::rust::Vec<::rpmostreecxx::StringMapping> *ptr) noexcept;
::std::size_t cxxbridge1$rust_vec$rpmostreecxx$StringMapping$capacity(const ::rust::Vec<::rpmostreecxx::StringMapping> *ptr) noexcept;
const ::rpmostreecxx::StringMapping *cxxbridge1$rust_vec$rpmostreecxx$StringMapping$data(const ::rust::Vec<::rpmostreecxx::StringMapping> *ptr) noexcept;
void cxxbridge1$rust_vec$rpmostreecxx$StringMapping$reserve_total(::rust::Vec<::rpmostreecxx::StringMapping> *ptr, ::std::size_t cap) noexcept;
void cxxbridge1$rust_vec$rpmostreecxx$StringMapping$set_len(::rust::Vec<::rpmostreecxx::StringMapping> *ptr, ::std::size_t len) noexcept;
::std::size_t cxxbridge1$rust_vec$rpmostreecxx$StringMapping$stride() noexcept;
#endif // CXXBRIDGE1_RUST_VEC_rpmostreecxx$StringMapping

#ifndef CXXBRIDGE1_RUST_BOX_rpmostreecxx$PasswdDB
#define CXXBRIDGE1_RUST_BOX_rpmostreecxx$PasswdDB
::rpmostreecxx::PasswdDB *cxxbridge1$box$rpmostreecxx$PasswdDB$alloc() noexcept;
void cxxbridge1$box$rpmostreecxx$PasswdDB$dealloc(::rpmostreecxx::PasswdDB *) noexcept;
void cxxbridge1$box$rpmostreecxx$PasswdDB$drop(::rust::Box<::rpmostreecxx::PasswdDB> *ptr) noexcept;
#endif // CXXBRIDGE1_RUST_BOX_rpmostreecxx$PasswdDB
} // extern "C"

namespace rust {
inline namespace cxxbridge1 {
template <>
::rpmostreecxx::TempEtcGuard *Box<::rpmostreecxx::TempEtcGuard>::allocation::alloc() noexcept {
  return cxxbridge1$box$rpmostreecxx$TempEtcGuard$alloc();
}
template <>
void Box<::rpmostreecxx::TempEtcGuard>::allocation::dealloc(::rpmostreecxx::TempEtcGuard *ptr) noexcept {
  cxxbridge1$box$rpmostreecxx$TempEtcGuard$dealloc(ptr);
}
template <>
void Box<::rpmostreecxx::TempEtcGuard>::drop() noexcept {
  cxxbridge1$box$rpmostreecxx$TempEtcGuard$drop(this);
}
template <>
Vec<::rpmostreecxx::StringMapping>::Vec() noexcept {
  cxxbridge1$rust_vec$rpmostreecxx$StringMapping$new(this);
}
template <>
void Vec<::rpmostreecxx::StringMapping>::drop() noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$drop(this);
}
template <>
::std::size_t Vec<::rpmostreecxx::StringMapping>::size() const noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$len(this);
}
template <>
::std::size_t Vec<::rpmostreecxx::StringMapping>::capacity() const noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$capacity(this);
}
template <>
const ::rpmostreecxx::StringMapping *Vec<::rpmostreecxx::StringMapping>::data() const noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$data(this);
}
template <>
void Vec<::rpmostreecxx::StringMapping>::reserve_total(::std::size_t cap) noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$reserve_total(this, cap);
}
template <>
void Vec<::rpmostreecxx::StringMapping>::set_len(::std::size_t len) noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$set_len(this, len);
}
template <>
::std::size_t Vec<::rpmostreecxx::StringMapping>::stride() noexcept {
  return cxxbridge1$rust_vec$rpmostreecxx$StringMapping$stride();
}
template <>
::rpmostreecxx::PasswdDB *Box<::rpmostreecxx::PasswdDB>::allocation::alloc() noexcept {
  return cxxbridge1$box$rpmostreecxx$PasswdDB$alloc();
}
template <>
void Box<::rpmostreecxx::PasswdDB>::allocation::dealloc(::rpmostreecxx::PasswdDB *ptr) noexcept {
  cxxbridge1$box$rpmostreecxx$PasswdDB$dealloc(ptr);
}
template <>
void Box<::rpmostreecxx::PasswdDB>::drop() noexcept {
  cxxbridge1$box$rpmostreecxx$PasswdDB$drop(this);
}
} // namespace cxxbridge1
} // namespace rust
